﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;
using System.Text.RegularExpressions;
using System.Diagnostics;

namespace CsFileCopy
{
    public partial class MainWindow : Form
    {
        /// <summary>
        /// CsFileCopyアプリケーションの設定情報
        /// </summary>
        private ApplicationSettings appSettings = new ApplicationSettings();

        /// <summary>
        /// 「使い方」ダイアログ
        /// </summary>
        private UsageDialog usageDialog = new UsageDialog();

        /// <summary>
        /// 抽出結果テーブル
        /// </summary>
        private DataTable resultTable;

        /// <summary>
        /// 抽出元フォルダー
        /// </summary>
        private string extractDirectory = "";

        /// <summary>
        /// 抽出先フォルダー
        /// </summary>
        private string destinationDirectory = "";

        /// <summary>
        /// 抽出実行スレッド
        /// </summary>
        private Thread extractThread;

        /// <summary>
        /// ファイルコピー実行スレッド
        /// </summary>
        private Thread copyThread;

        /// <summary>
        /// ファイルコピースレッドを中止するかどうか
        /// </summary>
        private bool requestStopCopyThread = false;

        /// <summary>
        /// アプリケーションの設定情報
        /// </summary>
        public ApplicationSettings AppSettings
        {
            get
            {
                return appSettings;
            }
        }

        public MainWindow()
        {
            InitializeComponent();
        }

        /// <summary>
        /// アプリケーションロード時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void MainWindow_Load(object sender, EventArgs e)
        {
            // アプリケーションの設定情報のロード
            if (appSettings.LoadAppSettings())
            {
                // 抽出元フォルダーリストの復元
                foreach (string dir in appSettings.ExtractDirectories)
                {
                    extractDirectoryComboBox.Items.Add(dir);
                }

                extractDirectoryComboBox.Text = appSettings.LastBrowseExtractDirectory;

                // 抽出対象ファイルリストの復元
                foreach (string file in appSettings.ExtractFiles)
                {
                    extractFileComboBox.Items.Add(file);
                }

                extractFileComboBox.Text = appSettings.LastInputedExtractFiles;

                // 抽出除外ファイルリストの復元
                foreach (string file in appSettings.ExceptFiles)
                {
                    exceptFileComboBox.Items.Add(file);
                }

                exceptFileComboBox.Text = appSettings.LastInputedExceptFiles;

                // 抽出先フォルダーリストの復元
                foreach (string dir in appSettings.DestinationDirectories)
                {
                    destinationDirectoryComboBox.Items.Add(dir);
                }

                destinationDirectoryComboBox.Text = appSettings.LastBrowseDestinationDirectory;


                // ウィンドウ幅の復元
                int winWidth = appSettings.WindowWidth;

                if (winWidth > 0)
                {
                    Width = winWidth;
                }

                // ウィンドウの高さの復元
                int winHeight = appSettings.WindowHeight;

                if (winHeight > 0)
                {
                    Height = winHeight;
                }

                // ウィンドウ位置の復元
                int winX = appSettings.WindowX;
                int winY = appSettings.WindowY;

                if (winX > 0 && winY > 0)
                {
                    Location = new Point(winX, winY);
                }

                // 最大化状態の設定
                if (appSettings.IsMaximizedOnTerminate)
                {
                    WindowState = FormWindowState.Maximized;
                }

                // サブフォルダーを検索対象とするかどうかの設定を復元
                subFolderCheckBox.Checked = appSettings.SearchSubFolder;

                // 各ラジオボタンの選択状態の復元
                fromToRadioButton.Checked = appSettings.FromToRadioChecked;
                beforeRadioButton.Checked = appSettings.BeforeRadioChecked;
                afterRadioButton.Checked = appSettings.AfterRadioChecked;
                excludeUpdateTimeRadio.Checked = appSettings.ExcludeUpdateTimeRadioChecked;
            }
        }

        /// <summary>
        /// フォームが最初に表示された時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void MainWindow_Shown(object sender, EventArgs e)
        {
            // コンボボックスのテキストを未選択にする
            extractDirectoryComboBox.SelectionLength = 0;
            extractFileComboBox.SelectionLength = 0;
            exceptFileComboBox.SelectionLength = 0;
            destinationDirectoryComboBox.SelectionLength = 0;
        }

        /// <summary>
        /// フォームがアクティブ化された時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void MainWindow_Activated(object sender, EventArgs e)
        {
            // コンボボックスのテキストを未選択にする
            extractDirectoryComboBox.SelectionLength = 0;
            extractFileComboBox.SelectionLength = 0;
            exceptFileComboBox.SelectionLength = 0;
            destinationDirectoryComboBox.SelectionLength = 0;
        }

        /// <summary>
        /// ウィンドウが閉じられるときの処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void MainWindow_FormClosing(object sender, FormClosingEventArgs e)
        {
            // 実行中の処理を停止
            StopOperation();

            // ウィンドウが通常表示の場合はサイズと位置を保存
            if (WindowState == FormWindowState.Normal)
            {
                appSettings.WindowHeight = Height;
                appSettings.WindowWidth = Width;
                appSettings.WindowX = Location.X;
                appSettings.WindowY = Location.Y;
            }

            // ウィンドウが最大化状態の場合は最大化状態フラグをセット
            if (WindowState == FormWindowState.Maximized)
            {
                appSettings.IsMaximizedOnTerminate = true;
            }
            else
            {
                appSettings.IsMaximizedOnTerminate = false;
            }

            // コンボボックスの内容を保存
            appSettings.LastBrowseExtractDirectory = extractDirectoryComboBox.Text;
            appSettings.LastInputedExtractFiles = extractFileComboBox.Text;
            appSettings.LastInputedExceptFiles = exceptFileComboBox.Text;
            appSettings.LastBrowseDestinationDirectory = destinationDirectoryComboBox.Text;

            // チェックボックスの設定値の保存
            appSettings.SearchSubFolder = subFolderCheckBox.Checked;

            // ラジオボタンの内容を保存
            appSettings.FromToRadioChecked = fromToRadioButton.Checked;
            appSettings.BeforeRadioChecked = beforeRadioButton.Checked;
            appSettings.AfterRadioChecked = afterRadioButton.Checked;
            appSettings.ExcludeUpdateTimeRadioChecked = excludeUpdateTimeRadio.Checked;

            appSettings.SaveAppSettings();
        }

        /// <summary>
        /// 入力値をチェックします。
        /// </summary>
        /// <returns>チェックOKの場合はtrue、NGの場合はfalse</returns>
        private bool InputForExtractIsOk()
        {
            if (extractDirectoryComboBox.Text == "")
            {
                MessageUtil.ShowAlert("抽出元フォルダーを指定してください。");
                return false;
            }

            try
            {
                DirectoryInfo di = new DirectoryInfo(extractDirectoryComboBox.Text);

                if (!di.Exists)
                {
                    MessageUtil.ShowAlert("抽出元フォルダーが存在しません。");
                    return false;
                }
            }
            catch (ArgumentException e)
            {
                Log.GetInstance().Warn(e.Message);
                MessageUtil.ShowAlert("抽出元フォルダーが無効です。");
                return false;
            }

            return true;
        }

        /// <summary>
        /// 入力値をチェックします。
        /// </summary>
        /// <returns>チェックOKの場合はtrue、NGの場合はfalse</returns>
        private bool InputForCopyIsOk()
        {
            if (destinationDirectoryComboBox.Text == "")
            {
                MessageUtil.ShowAlert("抽出先フォルダーを指定してください。");
                return false;
            }

            DirectoryInfo di = null;

            try
            {
                di = new DirectoryInfo(destinationDirectoryComboBox.Text);

                if (!di.Exists)
                {
                    MessageUtil.ShowAlert("抽出先フォルダーが存在しません。");
                    return false;
                }
            }
            catch (ArgumentException e)
            {
                Log.GetInstance().Warn(e.Message);
                MessageUtil.ShowAlert("抽出先フォルダーが無効です。");
                return false;
            }

            if (extractDirectory == destinationDirectoryComboBox.Text)
            {
                MessageUtil.ShowAlert("抽出元フォルダーと抽出先フォルダーが同じです。");
                return false;
            }

            if (resultTable == null || resultTable.Rows.Count == 0)
            {
                MessageUtil.ShowAlert("抽出結果がありません。");
                return false;
            }

            bool copyFileExists = false;

            foreach (DataRow row in resultTable.Rows)
            {
                bool copy = (bool)row[0];
                if (copy)
                {
                    copyFileExists = true;
                    break;
                }
            }

            if (!copyFileExists)
            {
                MessageUtil.ShowAlert("コピー対象のファイルが選択されていません。");
                return false;
            }

            FileInfo[] files = di.GetFiles();
            DirectoryInfo[] directories = di.GetDirectories();

            if (files.Length > 0 || directories.Length > 0)
            {
                DialogResult result = MessageUtil.ShowQuestion("抽出先フォルダーにファイルが存在します。\nコピーしてもよろしいですか？");

                if (result != DialogResult.Yes)
                {
                    return false;
                }
            }

            return true;
        }

        /// <summary>
        /// 抽出元フォルダーを返します。
        /// </summary>
        /// <returns>抽出元フォルダー</returns>
        private string GetExtractDirectory()
        {
            return extractDirectoryComboBox.Text;
        }

        /// <summary>
        /// 抽出対象ファイルを返します。
        /// </summary>
        /// <returns>抽出対象ファイル</returns>
        private string GetExtractFiles()
        {
            return extractFileComboBox.Text;
        }

        /// <summary>
        /// 抽出除外ファイルを返します。
        /// </summary>
        /// <returns>抽出除外ファイル</returns>
        private string GetExceptFiles()
        {
            return exceptFileComboBox.Text;
        }

        /// <summary>
        /// 抽出先フォルダーを返します。
        /// </summary>
        /// <returns>抽出先フォルダー</returns>
        private string GetDestinationDirectory()
        {
            return destinationDirectoryComboBox.Text;
        }

        /// <summary>
        /// 「サブフォルダーも対象」チェックボックスの状態を返します。
        /// </summary>
        /// <returns>チェックボックスの状態</returns>
        private bool GetSearchSubFolder()
        {
            return subFolderCheckBox.Checked;
        }

        /// <summary>
        /// 「開始～終了」ラジオボタンがチェックされているかどうかを返します。
        /// </summary>
        /// <returns>ラジオボタンがチェックされているかどうか</returns>
        private bool FromToRadioChecked()
        {
            return fromToRadioButton.Checked;
        }

        /// <summary>
        /// 「以前」ラジオボタンがチェックされているかどうかを返します。
        /// </summary>
        /// <returns>ラジオボタンがチェックされているかどうか</returns>
        private bool BeforeRadioChecked()
        {
            return beforeRadioButton.Checked;
        }

        /// <summary>
        /// 「以降」ラジオボタンがチェックされているかどうかを返します。
        /// </summary>
        /// <returns>ラジオボタンがチェックされているかどうか</returns>
        private bool AfterRadioChecked()
        {
            return afterRadioButton.Checked;
        }

        /// <summary>
        /// 「更新日時を抽出条件に含めない」ラジオボタンがチェックされているかどうかを返します。
        /// </summary>
        /// <returns>ラジオボタンがチェックされているかどうか</returns>
        private bool ExcludeUpdateTimeRadioChecked()
        {
            return excludeUpdateTimeRadio.Checked;
        }

        /// <summary>
        /// 開始日時を返します。
        /// </summary>
        /// <returns>開始日時</returns>
        private DateTime GetFromDate()
        {
            DateTime from = new DateTime(fromDate.Value.Year, fromDate.Value.Month, fromDate.Value.Day,
                (int)fromHour.Value, (int)fromMinute.Value, (int)fromSecond.Value);

            return from;
        }

        /// <summary>
        /// 終了日時を返します。
        /// </summary>
        /// <returns>終了日時</returns>
        private DateTime GetToDate()
        {
            DateTime to = new DateTime(toDate.Value.Year, toDate.Value.Month, toDate.Value.Day,
                (int)toHour.Value, (int)toMinute.Value, (int)toSecond.Value);

            return to;
        }

        /// <summary>
        /// 「以前」日時を返します。
        /// </summary>
        /// <returns>以前日時</returns>
        private DateTime GetBeforeDate()
        {
            DateTime before = new DateTime(beforeDate.Value.Year, beforeDate.Value.Month, beforeDate.Value.Day,
                (int)beforeHour.Value, (int)beforeMinute.Value, (int)beforeSecond.Value);

            return before;
        }

        /// <summary>
        /// 「以降」日時を返します。
        /// </summary>
        /// <returns>以降日時</returns>
        private DateTime GetAfterDate()
        {
            DateTime after = new DateTime(afterDate.Value.Year, afterDate.Value.Month, afterDate.Value.Day,
                (int)afterHour.Value, (int)afterMinute.Value, (int)afterSecond.Value);

            return after;
        }

        /// <summary>
        /// ファイル抽出を実行します。
        /// </summary>
        /// <remarks>
        /// 抽出処理のエントリーポイントです。
        /// </remarks>
        private void ExtractFile()
        {
            if (!InputForExtractIsOk())
            {
                return;
            }

            // 入力履歴の保存
            SaveInputHistory();

            // 抽出元フォルダーの保存
            extractDirectory = extractDirectoryComboBox.Text;

            // ツールバーのボタン、メニューを無効にする
            DisableButtonsAndMenus();

            BindDataTable(null);
            resultTable = CreateDataTable();

            ThreadStart worker = new ThreadStart(ExecuteFileExtract);
            extractThread = new Thread(worker);
            extractThread.Start();
        }

        /// <summary>
        /// ファイル抽出処理を実行します。
        /// </summary>
        /// <remarks>
        /// 抽出処理の実行スレッドです。
        /// </remarks>
        private void ExecuteFileExtract()
        {
            GC.Collect();

            // 抽出結果一覧の出力行数を初期化
            Invoke(new Action<int>(SetOuputLineCount), new object[] { 0 });

            // ファイルの合計サイズを初期化
            Invoke(new MethodInvoker(CalculateTotalFileSize));

            FileRetriever retriever = new FileRetriever();

            retriever.Log = Log.GetInstance();
            retriever.SearchFolderChanged += new FileRetrieveEventHandler(this.fileRetriever_SearchFolderChanged);
            retriever.AddingFileToList += new FileRetrieveEventHandler(this.fileRetriever_AddingFileToList);

            List<string> fileList = new List<string>();

            string extractDirectory = (string)Invoke(new Func<string>(GetExtractDirectory));

            bool searchSubFolder = (bool)Invoke(new Func<bool>(GetSearchSubFolder));

            if (searchSubFolder)
            {
                retriever.GetAllFilesRecursive(extractDirectory, "*.*", ref fileList);
            }
            else
            {
                retriever.GetAllFilesInDirectory(extractDirectory, "*.*", ref fileList);
            }

            foreach (string s in fileList)
            {
                Invoke(new Action<string>(AddExtractResult), new object[] { s });
            }

            SetStatusBarText("検索終了");

            // データグリッドにデータテーブルを連結
            Invoke(new Action<DataTable>(BindDataTable), new object[] { resultTable });

            if (fileList.Count > 0)
            {
                // データグリッドの内容列の幅を調整
                Invoke(new MethodInvoker(SetColumnWidth));

                // 抽出結果一覧のセルの選択をクリア
                Invoke(new MethodInvoker(ClearCellSelection));

                // 抽出結果一覧のセルのソートを不可に設定
                Invoke(new MethodInvoker(SetNotSortable));

                // 抽出結果一覧の出力行数を設定
                Invoke(new Action<int>(SetOuputLineCount), new object[] { fileList.Count });

                // ファイルの合計サイズを計算
                Invoke(new MethodInvoker(CalculateTotalFileSize));
            }

            // ツールバーのボタン、メニューを有効にする。
            Invoke(new MethodInvoker(EnableButtonsAndMenus));
        }

        /// <summary>
        /// ファイルコピーを実行します。
        /// </summary>
        /// <remarks>
        /// コピー処理のエントリーポイントです。
        /// </remarks>
        private void CopyFile()
        {
            if (!InputForCopyIsOk())
            {
                return;
            }

            // 入力履歴の保存
            SaveInputHistory();

            // 抽出先フォルダーの保存
            destinationDirectory = destinationDirectoryComboBox.Text;

            // データグリッドビューの編集状態を解除
            dataGridView1.EndEdit();

            // ツールバーのボタン、メニューを無効にする
            DisableButtonsAndMenus();

            ThreadStart worker = new ThreadStart(ExecuteFileCopy);
            copyThread = new Thread(worker);
            copyThread.Start();
        }

        /// <summary>
        /// ファイルコピー処理を実行します。
        /// </summary>
        /// <remarks>
        /// コピー処理の実行スレッドです。
        /// </remarks>
        private void ExecuteFileCopy()
        {
            SetStatusBarText("コピー中...");

            // プログレスバーを表示
            Invoke(new MethodInvoker(ShowProgressBar));

            // プログレスバーの最大、最小値をセット
            Invoke(new Action<int, int>(SetMinimumAndMaximumOfProgressBar), 0, resultTable.Rows.Count);

            // プログレスバーの値をリセット
            Invoke(new Action<int>(SetCurrentValueOfProgressBar), 0);

            try
            {
                foreach (DataRow row in resultTable.Rows)
                {
                    // コピー終了フラグが立っている場合は処理終了
                    if (requestStopCopyThread)
                    {
                        requestStopCopyThread = false;
                        Log.GetInstance().Error("Stopping copy thread 1 ...");
                        return;
                    }

                    bool copy = (bool)row[0];
                    string fileName = (string)row[1];
                    string directory = (string)row[2];

                    // プログレスバーの値を1増やす
                    Invoke(new Action<int>(IncrementProgressBar), 1);

                    // コピー対象でない場合はスキップ
                    if (!copy)
                    {
                        continue;
                    }

                    string newDirectory = directory.Replace(extractDirectory, destinationDirectory);

                    DirectoryInfo di = new DirectoryInfo(newDirectory);

                    // コピー先にフォルダーが存在しない場合は作成する
                    if (!di.Exists)
                    {
                        di.Create();
                    }

                    string from = directory + "\\" + fileName;
                    string to = newDirectory + "\\" + fileName;

                    // ファイルコピー
                    File.Copy(from, to, true);

                    // コピー終了フラグが立っている場合は処理終了
                    if (requestStopCopyThread)
                    {
                        requestStopCopyThread = false;
                        Log.GetInstance().Error("Stopping copy thread 2 ...");
                        return;
                    }
                }

                MessageUtil.ShowInfo("ファイルをコピーしました。");
            }
            catch (ThreadAbortException e)
            {
                Log.GetInstance().Error("ファイルコピーエラー①");
                Log.GetInstance().Error(e.Message);
                Log.GetInstance().Error(e.StackTrace);
                if (e.InnerException != null)
                {
                    Log.GetInstance().Error(e.InnerException.Message);
                    Log.GetInstance().Error(e.InnerException.StackTrace);
                }
            }
            catch (Exception e)
            {
                MessageUtil.ShowAlert("ファイルのコピー中にエラーが発生しました。\n" + e.Message);
                Log.GetInstance().Error("ファイルコピーエラー②");
                Log.GetInstance().Error(e.Message);
                Log.GetInstance().Error(e.StackTrace);
                if (e.InnerException != null)
                {
                    Log.GetInstance().Error(e.InnerException.Message);
                    Log.GetInstance().Error(e.InnerException.StackTrace);
                }
            }

            // プログレスバーを非表示にする
            Invoke(new MethodInvoker(HideProgressBar));

            SetStatusBarText("コピー終了");

            // ツールバーのボタン、メニューを有効にする。
            Invoke(new MethodInvoker(EnableButtonsAndMenus));
        }

        /// <summary>
        /// プログレスバーの最大、最小値を設定します。
        /// </summary>
        /// <param name="min">最小値</param>
        /// <param name="max">最大値</param>
        private void SetMinimumAndMaximumOfProgressBar(int min, int max)
        {
            toolStripProgressBar1.Minimum = min;
            toolStripProgressBar1.Maximum = max;
        }

        /// <summary>
        /// プログレスバーの現在値を設定します。
        /// </summary>
        /// <param name="value">設定する値</param>
        private void SetCurrentValueOfProgressBar(int value)
        {
            toolStripProgressBar1.Value = value;
        }

        /// <summary>
        /// プログレスバーの現在値を増やします。
        /// </summary>
        /// <param name="value">増やす値</param>
        private void IncrementProgressBar(int value)
        {
            toolStripProgressBar1.Increment(value);
        }

        /// <summary>
        /// プログレスバーを表示します。
        /// </summary>
        private void ShowProgressBar()
        {
            toolStripProgressBar1.Visible = true;
        }

        /// <summary>
        /// プログレスバーを隠します。
        /// </summary>
        private void HideProgressBar()
        {
            toolStripProgressBar1.Visible = false;
        }

        /// <summary>
        /// 抽出元フォルダーの参照ボタンクリック時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void extractDirectoryBrowseButton_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog folderBrowserDialog1 = new FolderBrowserDialog();
            folderBrowserDialog1.Description = "抽出元フォルダーを選択してください。";

            try
            {
                DirectoryInfo di = new DirectoryInfo(extractDirectoryComboBox.Text);
                if (di.Exists)
                {
                    folderBrowserDialog1.SelectedPath = di.FullName;
                }
            }
            catch (ArgumentException ae)
            {
                Log.GetInstance().Warn(ae.Message);
            }

            DialogResult result = FolderBrowserLauncher.ShowFolderBrowser(folderBrowserDialog1, this);

            if (result == DialogResult.OK)
            {
                extractDirectoryComboBox.Text = folderBrowserDialog1.SelectedPath;
            }
        }

        /// <summary>
        /// 抽出先フォルダーの参照ボタンクリック時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void destinationDirectoryBrowseButton_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog folderBrowserDialog1 = new FolderBrowserDialog();
            folderBrowserDialog1.Description = "抽出先フォルダーを選択してください。";

            try
            {
                DirectoryInfo di = new DirectoryInfo(destinationDirectoryComboBox.Text);
                if (di.Exists)
                {
                    folderBrowserDialog1.SelectedPath = di.FullName;
                }
            }
            catch (ArgumentException ae)
            {
                Log.GetInstance().Warn(ae.Message);
            }

            DialogResult result = FolderBrowserLauncher.ShowFolderBrowser(folderBrowserDialog1, this);

            if (result == DialogResult.OK)
            {
                destinationDirectoryComboBox.Text = folderBrowserDialog1.SelectedPath;
            }
        }

        /// <summary>
        /// ステータスバーのテキストを設定します。
        /// </summary>
        /// <param name="s">テキスト</param>
        public void SetStatusBarText(string s)
        {
            Invoke(new Action<string>(DoSetStatusBarText), new object[] { s });
        }

        /// <summary>
        /// ステータスバーのテキストを設定します。
        /// </summary>
        /// <param name="s">テキスト</param>
        private void DoSetStatusBarText(string s)
        {
            statusLabel.Text = s;
        }

        /// <summary>
        /// 抽出実行ボタン、コピーボタン、結果を出力ボタン等を無効にし、中止ボタンを有効にします。
        /// </summary>
        private void DisableButtonsAndMenus()
        {
            extractButton.Enabled = false;
            copyButton.Enabled = false;
            stopButton.Enabled = true;
            outputResultButton.Enabled = false;
            extractMenuItem.Enabled = false;
            fileCopyMenuItem.Enabled = false;
            outputResultMenuItem.Enabled = false;
            addToFavoriteMenuItem.Enabled = false;
            readFavoriteMenuItem.Enabled = false;
        }

        /// <summary>
        /// 抽出実行ボタン、コピーボタン、結果を出力ボタン等を有効にし、中止ボタンを無効にします。
        /// </summary>
        private void EnableButtonsAndMenus()
        {
            extractButton.Enabled = true;
            copyButton.Enabled = true;
            stopButton.Enabled = false;
            outputResultButton.Enabled = true;
            extractMenuItem.Enabled = true;
            fileCopyMenuItem.Enabled = true;
            outputResultMenuItem.Enabled = true;
            addToFavoriteMenuItem.Enabled = true;
            readFavoriteMenuItem.Enabled = true;
        }

        /// <summary>
        /// 空のデータテーブルを作成します。
        /// </summary>
        /// <returns></returns>
        private DataTable CreateDataTable()
        {
            DataTable dt = new DataTable("DataTable1");
            dt.Columns.Add("コピー", typeof(bool));
            dt.Columns.Add("ファイル");
            dt.Columns.Add("場所");
            dt.Columns.Add("更新日時");
            dt.Columns.Add("ファイルサイズ");
            dt.Columns[1].ReadOnly = true;
            dt.Columns[2].ReadOnly = true;
            dt.Columns[3].ReadOnly = true;
            dt.Columns[4].ReadOnly = true;

            return dt;
        }

        /// <summary>
        /// データグリッドにデータテーブルを連結します。
        /// </summary>
        /// <param name="dt">データテーブル</param>
        private void BindDataTable(DataTable dt)
        {
            if (dt != null && dt.Rows.Count > 0)
            {
                dataGridView1.DataSource = dt;
            }
            else
            {
                dataGridView1.DataSource = null;
            }
        }

        /// <summary>
        /// DataGridの内容列の幅をTextの幅に合わせて変更する
        /// </summary>
        private void SetColumnWidth()
        {
            dataGridView1.Columns[0].Width = 45;
            dataGridView1.Columns[1].Width = 200;
            dataGridView1.AutoResizeColumn(2);
            dataGridView1.AutoResizeColumn(3);
            dataGridView1.AutoResizeColumn(4);
            dataGridView1.Columns[4].DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight;
        }

        /// <summary>
        /// DataGridのカラムをソート不可に設定します。
        /// </summary>
        private void SetNotSortable()
        {
            // カラムのソートを不可に設定
            dataGridView1.Columns[0].SortMode = DataGridViewColumnSortMode.NotSortable;
        }

        /// <summary>
        /// 結果一覧のセルの選択をクリアします。
        /// </summary>
        private void ClearCellSelection()
        {
            dataGridView1.ClearSelection();
        }

        /// <summary>
        /// 抽出結果一覧の出力行数を設定します。
        /// </summary>
        /// <param name="count">出力行数</param>
        private void SetOuputLineCount(int count)
        {
            countLabel.Text = count.ToString();
        }

		/// <summary>
		/// 抽出結果の一行を一覧に追加します。
		/// </summary>
		/// <param name="s">検索結果の一行</param>
        private void AddExtractResult(string s)
        {
            FileInfo fi = new FileInfo(s);

            double fileSizeDouble = Convert.ToDouble(fi.Length) / (double)1024.0;
            string fileSizeString = String.Format("{0:000000000.00}", fileSizeDouble);

            resultTable.Rows.Add(new object[] { true, fi.Name, fi.DirectoryName, fi.LastWriteTime.ToString("yyyy/MM/dd HH:mm:ss"), fileSizeString });
        }

        /// <summary>
        /// ファイル抽出クラスで検索対象のフォルダーが変更されたときの処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void fileRetriever_SearchFolderChanged(object sender, FileRetrieveEventArgs e)
        {
            SetStatusBarText("検索中...  " + e.SearchFolder);
        }

        /// <summary>
        /// ファイル抽出クラスでリストにファイルが追加されるときの処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void fileRetriever_AddingFileToList(object sender, FileRetrieveEventArgs e)
        {
            SetStatusBarText("検索中...  " + e.FileName);

            // 抽出対象ファイルのリスト
            string extractFiles = (string)Invoke(new Func<string>(GetExtractFiles));
            string[] extractFileList = extractFiles.Split(new char[]{';'});
            // ファイル名のTrim
            for (int i = 0; i < extractFileList.Length; i++)
            {
                extractFileList[i] = extractFileList[i].Trim();
            }

            // 抽出除外ファイルのリスト
            string exceptFiles = (string)Invoke(new Func<string>(GetExceptFiles));
            string[] exceptFileList = exceptFiles.Split(new char[] { ';' });
            // ファイル名のTrim
            for (int i = 0; i < exceptFileList.Length; i++)
            {
                exceptFileList[i] = exceptFileList[i].Trim();
            }

            // 正規表現のオプション
            RegexOptions regexOptions = RegexOptions.None;

            // ファイル名の大文字小文字を区別しない場合
            if (appSettings.IgnoreCaseRadioChecked)
            {
                regexOptions = RegexOptions.IgnoreCase;
            }

            // 抽出対象ファイルも抽出除外ファイルも指定されていない場合はすべてのファイルを追加する
            if (extractFiles.Trim() == "" && exceptFiles.Trim() == "")
            {
                e.AddToFile = IsInAddDateRange(e.FileName);
                return;
            }
            // 抽出対象ファイルと抽出除外ファイルが指定されている場合
            else if (extractFiles.Trim() != "" && exceptFiles.Trim() != "")
            {
                // 抽出対象ファイルのリストに存在するかどうかをチェック
                foreach (string extractFile in extractFileList)
                {
                    Regex regex1 = new Regex(REConverter.GetRegularExpression(extractFile), regexOptions);

                    // 抽出対象
                    if (regex1.IsMatch(e.FileName))
                    {
                        // 抽出除外ファイルのリストに存在するかどうかをチェック
                        foreach (string exceptFile in exceptFileList)
                        {
                            Regex regex2 = new Regex(REConverter.GetRegularExpression(exceptFile), regexOptions);

                            // 除外対象
                            if (regex2.IsMatch(e.FileName))
                            {
                                // 除外対象の場合は追加しない
                                e.AddToFile = false;
                                return;
                            }
                        }

                        e.AddToFile = IsInAddDateRange(e.FileName);
                        return;
                    }
                }

                // 抽出対象リストに存在しない場合は追加しない
                e.AddToFile = false;
                return;
            }
            // 抽出対象ファイルが指定されている場合
            else if (extractFiles.Trim() != "" && exceptFiles.Trim() == "")
            {
                // 抽出対象ファイルのリストに存在するかどうかをチェック
                foreach (string extractFile in extractFileList)
                {
                    Regex regex1 = new Regex(REConverter.GetRegularExpression(extractFile), regexOptions);

                    // 抽出対象
                    if (regex1.IsMatch(e.FileName))
                    {
                        e.AddToFile = IsInAddDateRange(e.FileName);
                        return;
                    }
                }

                // 抽出対象リストに存在しない場合は追加しない
                e.AddToFile = false;
                return;
            }
            // 抽出除外ファイルが指定されている場合
            else if (extractFiles.Trim() == "" && exceptFiles.Trim() != "")
            {
                // 抽出除外ファイルのリストに存在するかどうかをチェック
                foreach (string exceptFile in exceptFileList)
                {
                    Regex regex = new Regex(REConverter.GetRegularExpression(exceptFile), regexOptions);

                    // 除外対象
                    if (regex.IsMatch(e.FileName))
                    {
                        e.AddToFile = false;
                        return;
                    }
                }

                // 除外対象リストに存在しない場合は追加する
                e.AddToFile = IsInAddDateRange(e.FileName);
                return;
            }
        }

        /// <summary>
        /// ファイルの更新日時が抽出対象の日時に含まれているかどうかを返します。
        /// </summary>
        /// <returns>抽出対象に含まれているかどうか</returns>
        private bool IsInAddDateRange(string fileName)
        {
            bool fromToRadioChecked = (bool)Invoke(new Func<bool>(FromToRadioChecked));
            bool beforeRadioChecked = (bool)Invoke(new Func<bool>(BeforeRadioChecked));
            bool afterRadioChecked = (bool)Invoke(new Func<bool>(AfterRadioChecked));
            bool excludeUpdateTimeRadioChecked = (bool)Invoke(new Func<bool>(ExcludeUpdateTimeRadioChecked));

            FileInfo fi = new FileInfo(fileName);

            // 開始終了が選択されている場合
            if (fromToRadioChecked)
            {
                DateTime from = (DateTime)Invoke(new Func<DateTime>(GetFromDate));
                DateTime to = (DateTime)Invoke(new Func<DateTime>(GetToDate));

                if (from <= fi.LastWriteTime && fi.LastWriteTime <= to)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            // 以前が選択されている場合
            else if (beforeRadioChecked)
            {
                DateTime before = (DateTime)Invoke(new Func<DateTime>(GetBeforeDate));

                if (fi.LastWriteTime <= before)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            // 以降が選択されている場合
            else if (afterRadioChecked)
            {
                DateTime after = (DateTime)Invoke(new Func<DateTime>(GetAfterDate));

                if (after <= fi.LastWriteTime)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            // 更新日時を抽出条件に含めないが選択されている場合
            else if (excludeUpdateTimeRadioChecked)
            {
                return true;
            }

            return false;
        }

        /// <summary>
        /// 抽出元フォルダーの入力ボックスでキーが押された時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void extractDirectoryComboBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Enter)
            {
                ExtractFile();
                e.Handled = true;
            }
        }

        /// <summary>
        /// 抽出対象ファイルの入力ボックスでキーが押された時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void extractFileComboBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                ExtractFile();
                //e.Handled = true;
            }
        }

        /// <summary>
        /// 抽出対象ファイルの入力ボックスでキーが押された時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void extractFileComboBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Enter)
            {
                e.Handled = true;
            }
        }

        /// <summary>
        /// 抽出除外ファイルの入力ボックスでキーが押された時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void exceptFileComboBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                ExtractFile();
                //e.Handled = true;
            }
        }

        /// <summary>
        /// 抽出除外ファイルの入力ボックスでキーが押された時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void exceptFileComboBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Enter)
            {
                e.Handled = true;
            }
        }

        /// <summary>
        /// 抽出先フォルダーの入力ボックスでキーが押された時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void destinationDirectoryComboBox_KeyPress(object sender, KeyPressEventArgs e)
        {

        }

        /// <summary>
        /// ファイル抽出ボタンがクリックされた時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void extractButton_Click(object sender, EventArgs e)
        {
            ExtractFile();
        }

        /// <summary>
        /// ファイルコピーボタンが押された時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void copyButton_Click(object sender, EventArgs e)
        {
            CopyFile();
        }

        /// <summary>
        /// ファイル抽出メニューがクリックされた時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void extractMenuItem_Click(object sender, EventArgs e)
        {
            ExtractFile();
        }

        /// <summary>
        /// 終了メニューがクリックされた時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void exitMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        /// <summary>
        /// ファイルコピーメニューがクリックされた時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void fileCopyMenuItem_Click(object sender, EventArgs e)
        {
            CopyFile();
        }

        /// <summary>
        /// 中止ボタンがクリックされた時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void stopButton_Click(object sender, EventArgs e)
        {
            StopOperation();
            SetStatusBarText("中止しました");
            HideProgressBar();
            EnableButtonsAndMenus();
        }

        /// <summary>
        /// 実行中の処理を停止します。
        /// </summary>
        private void StopOperation()
        {
            if (extractThread != null && extractThread.IsAlive)
            {
                try
                {
                    extractThread.Abort();
                    extractThread = null;
                }
                catch (Exception e)
                {
                    Log.GetInstance().Error(e.Message);
                    Log.GetInstance().Error(e.StackTrace);
                }
            }

            if (copyThread != null && copyThread.IsAlive)
            {
                // コピースレッドの終了を要求
                requestStopCopyThread = true;

                // コピースレッドの終了を待機
                copyThread.Join();
            }
        }

        /// <summary>
        /// 検索条件に入力された情報を履歴に保存します。
        /// </summary>
        private void SaveInputHistory()
        {
            if (extractDirectoryComboBox.Text.Trim() != "")
            {
                // 入力されたフォルダーがまだ履歴にない場合
                if (!extractDirectoryComboBox.Items.Contains(extractDirectoryComboBox.Text))
                {
                    // 入力されたフォルダーを履歴に保存
                    appSettings.AddExtractDirectory(extractDirectoryComboBox.Text);

                    extractDirectoryComboBox.Items.Insert(0, extractDirectoryComboBox.Text);

                    // 履歴保持数の最大を超えた場合
                    if (extractDirectoryComboBox.Items.Count > ApplicationSettings.INPUT_HISTORY_MAX)
                    {
                        // 最後の要素を削除
                        extractDirectoryComboBox.Items.RemoveAt(
                            extractDirectoryComboBox.Items.Count - 1);
                    }
                }
                else // すでに履歴にある場合
                {
                    // 入力されたフォルダーを履歴に保存
                    appSettings.AddExtractDirectory(extractDirectoryComboBox.Text);

                    // 入力されたテキストを一時保存
                    string tempText = extractDirectoryComboBox.Text;

                    // 現在の履歴の中のインデックスを取得
                    int index = extractDirectoryComboBox.Items.IndexOf(extractDirectoryComboBox.Text);

                    // 現在の位置のテキストを削除
                    extractDirectoryComboBox.Items.RemoveAt(index);

                    // 履歴の先頭に追加
                    extractDirectoryComboBox.Items.Insert(0, tempText);

                    extractDirectoryComboBox.Text = tempText;
                }
            }

            if (extractFileComboBox.Text.Trim() != "")
            {
                // 入力されたファイル名がまだ履歴にない場合
                if (!extractFileComboBox.Items.Contains(extractFileComboBox.Text))
                {
                    // 入力されたファイル名を履歴に保存
                    appSettings.AddExtractFile(extractFileComboBox.Text);

                    extractFileComboBox.Items.Insert(0, extractFileComboBox.Text);

                    // 履歴保持数の最大を超えた場合
                    if (extractFileComboBox.Items.Count > ApplicationSettings.INPUT_HISTORY_MAX)
                    {
                        // 最後の要素を削除
                        extractFileComboBox.Items.RemoveAt(
                            extractFileComboBox.Items.Count - 1);
                    }
                }
                else // すでに履歴にある場合
                {
                    // 入力されたファイル名を履歴に保存
                    appSettings.AddExtractFile(extractFileComboBox.Text);

                    // 入力されたテキストを一時保存
                    string tempText = extractFileComboBox.Text;

                    // 現在の履歴の中のインデックスを取得
                    int index = extractFileComboBox.Items.IndexOf(extractFileComboBox.Text);

                    // 現在の位置のテキストを削除
                    extractFileComboBox.Items.RemoveAt(index);

                    // 履歴の先頭に追加
                    extractFileComboBox.Items.Insert(0, tempText);

                    extractFileComboBox.Text = tempText;
                }
            }

            if (exceptFileComboBox.Text.Trim() != "")
            {
                // 入力されたファイル名がまだ履歴にない場合
                if (!exceptFileComboBox.Items.Contains(exceptFileComboBox.Text))
                {
                    // 入力されたファイル名を履歴に保存
                    appSettings.AddExceptFile(exceptFileComboBox.Text);

                    exceptFileComboBox.Items.Insert(0, exceptFileComboBox.Text);

                    // 履歴保持数の最大を超えた場合
                    if (exceptFileComboBox.Items.Count > ApplicationSettings.INPUT_HISTORY_MAX)
                    {
                        // 最後の要素を削除
                        exceptFileComboBox.Items.RemoveAt(
                            exceptFileComboBox.Items.Count - 1);
                    }
                }
                else // すでに履歴にある場合
                {
                    // 入力されたファイル名を履歴に保存
                    appSettings.AddExceptFile(exceptFileComboBox.Text);

                    // 入力されたテキストを一時保存
                    string tempText = exceptFileComboBox.Text;

                    // 現在の履歴の中のインデックスを取得
                    int index = exceptFileComboBox.Items.IndexOf(exceptFileComboBox.Text);

                    // 現在の位置のテキストを削除
                    exceptFileComboBox.Items.RemoveAt(index);

                    // 履歴の先頭に追加
                    exceptFileComboBox.Items.Insert(0, tempText);

                    exceptFileComboBox.Text = tempText;
                }
            }

            if (destinationDirectoryComboBox.Text.Trim() != "")
            {
                // 入力されたフォルダーがまだ履歴にない場合
                if (!destinationDirectoryComboBox.Items.Contains(destinationDirectoryComboBox.Text))
                {
                    // 入力されたフォルダーを履歴に保存
                    appSettings.AddDestinationDirectory(destinationDirectoryComboBox.Text);

                    destinationDirectoryComboBox.Items.Insert(0, destinationDirectoryComboBox.Text);

                    // 履歴保持数の最大を超えた場合
                    if (destinationDirectoryComboBox.Items.Count > ApplicationSettings.INPUT_HISTORY_MAX)
                    {
                        // 最後の要素を削除
                        destinationDirectoryComboBox.Items.RemoveAt(
                            destinationDirectoryComboBox.Items.Count - 1);
                    }
                }
                else // すでに履歴にある場合
                {
                    // 入力されたフォルダーを履歴に保存
                    appSettings.AddDestinationDirectory(destinationDirectoryComboBox.Text);

                    // 入力されたテキストを一時保存
                    string tempText = destinationDirectoryComboBox.Text;

                    // 現在の履歴の中のインデックスを取得
                    int index = destinationDirectoryComboBox.Items.IndexOf(destinationDirectoryComboBox.Text);

                    // 現在の位置のテキストを削除
                    destinationDirectoryComboBox.Items.RemoveAt(index);

                    // 履歴の先頭に追加
                    destinationDirectoryComboBox.Items.Insert(0, tempText);

                    destinationDirectoryComboBox.Text = tempText;
                }
            }
        }

        /// <summary>
        /// バージョン情報メニューがクリックされた時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void versionMenuItem_Click(object sender, EventArgs e)
        {
            VersionDialog dialog = new VersionDialog();
            dialog.ShowDialog();
        }

        /// <summary>
        /// 使い方メニューがクリックされた時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void usageMenuItem_Click(object sender, EventArgs e)
        {
            if (usageDialog.IsDisposed)
            {
                usageDialog = new UsageDialog();
            }

            usageDialog.Show();
            usageDialog.BringToFront();
        }

        /// <summary>
        /// 結果をファイルに出力ボタンクリック時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void outputResultButton_Click(object sender, EventArgs e)
        {
            OutputResult();
        }

        /// <summary>
        /// 結果をファイルに出力メニューがクリックされた時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void outputResultMenuItem_Click(object sender, EventArgs e)
        {
            OutputResult();
        }

        /// <summary>
        /// 抽出結果をファイルに出力します。
        /// </summary>
        private void OutputResult()
        {
            if (resultTable == null || resultTable.Rows.Count == 0)
            {
                MessageUtil.ShowAlert("抽出結果がありません。");
                return;
            }

            ResultOutputDialog dialog = new ResultOutputDialog(this, dataGridView1);
            dialog.ShowDialog();
        }

        /// <summary>
        /// データグリッドビューへの編集を終了させます。
        /// </summary>
        public void EndDataGridViewEdit()
        {
            dataGridView1.EndEdit();
        }

        /// <summary>
        /// ツール＞オプション　メニューがクリックされた時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void optionMenuItem_Click(object sender, EventArgs e)
        {
            OptionDialog dialog = new OptionDialog(this);
            dialog.ShowDialog();
        }

        /// <summary>
        /// コンテキストメニューの項目がクリックされた時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void contextMenuStrip1_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            if (e.ClickedItem == copyMenuItem)
            {
                if (dataGridView1.SelectedCells.Count == 0)
                {
                    MessageUtil.ShowAlert("コピーするセルを選択してください。");
                    return;
                }

                // ファイルサイズが選択されている場合
                if (dataGridView1.SelectedCells[0].ColumnIndex == 4)
                {
                    // 文字列を数値に変換(先頭の0を除去する)
                    double value = Double.Parse(dataGridView1.SelectedCells[0].Value.ToString());

                    // 再び数値を文字列に変換する。(カンマ区切りで)
                    string fileSize = String.Format("{0:#,0.00}KB", value);

                    // クリップボードにコピー
                    Clipboard.SetDataObject(fileSize, true);
                }
                else
                {
                    // クリップボードに直接コピー
                    Clipboard.SetDataObject(dataGridView1.SelectedCells[0].Value, true);
                }
            }
        }

        /// <summary>
        /// 全選択ボタンがクリックされた時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void selectAllButton_Click(object sender, EventArgs e)
        {
            if (resultTable == null || resultTable.Rows.Count == 0)
            {
                MessageUtil.ShowAlert("抽出結果がありません。");
                return;
            }

            foreach (DataRow row in resultTable.Rows)
            {
                row[0] = true;
            }

            CalculateTotalFileSize();
        }

        /// <summary>
        /// 全選択解除ボタンがクリックされた時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void unselectAllButton_Click(object sender, EventArgs e)
        {
            if (resultTable == null || resultTable.Rows.Count == 0)
            {
                MessageUtil.ShowAlert("抽出結果がありません。");
                return;
            }

            foreach (DataRow row in resultTable.Rows)
            {
                row[0] = false;
            }

            CalculateTotalFileSize();
        }

        /// <summary>
        /// 抽出結果一覧のセルの表示時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void dataGridView1_CellFormatting(object sender, DataGridViewCellFormattingEventArgs e)
        {
            if (e.ColumnIndex == 4)
            {
                double fileSizeDouble =  Convert.ToDouble(e.Value.ToString());
                string fileSizeString = String.Format("{0:#,0.00}KB", fileSizeDouble);
                e.Value = fileSizeString;
            }
        }

        /// <summary>
        /// 選択されたファイルの合計サイズを計算します。
        /// </summary>
        private void CalculateTotalFileSize()
        {
            double totalFileSize = 0.0;

            if (resultTable != null)
            {
                foreach (DataRow row in resultTable.Rows)
                {
                    bool copy = (bool)row[0];
                    double fileSizeDouble = Convert.ToDouble(row[4]);
                    if (copy)
                    {
                        totalFileSize += fileSizeDouble;
                    }
                }
            }

            totalSizeLabel.Text = String.Format("{0:#,0.00}", totalFileSize);
        }

        /// <summary>
        /// 抽出結果一覧のセルの値が変化した時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void dataGridView1_CellValueChanged(object sender, DataGridViewCellEventArgs e)
        {
            if (e.ColumnIndex == 0)
            {
                CalculateTotalFileSize();
            }
        }

        /// <summary>
        /// 抽出結果一覧のセルの内容が変更される時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void dataGridView1_CurrentCellDirtyStateChanged(object sender, EventArgs e)
        {
            if (dataGridView1.CurrentCellAddress.X == 0)
            {
                dataGridView1.CommitEdit(DataGridViewDataErrorContexts.Commit);
            }
        }

        /// <summary>
        /// 「更新日時を抽出条件に含めない」ラジオボタンがクリックされた時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void excludeUpdateTimeRadio_CheckedChanged(object sender, EventArgs e)
        {
            if (excludeUpdateTimeRadio.Checked)
            {
                disableDateInputFields();
            }
        }

        /// <summary>
        /// 「更新日時(開始・終了)」指定ラジオボタンがクリックされた時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void fromToRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            if (fromToRadioButton.Checked)
            {
                enableDateInputFields();
            }
        }

        /// <summary>
        /// 「更新日時(以前)」指定ラジオボタンがクリックされた時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void beforeRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            if (beforeRadioButton.Checked)
            {
                enableDateInputFields();
            }
        }

        /// <summary>
        /// 「更新日時(以降)」指定ラジオボタンがクリックされた時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void afterRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            if (afterRadioButton.Checked)
            {
                enableDateInputFields();
            }
        }

        /// <summary>
        /// 更新日時の抽出条件指定フィールドを非活性にします。
        /// </summary>
        private void disableDateInputFields()
        {
            fromDate.Enabled = false;
            fromHour.Enabled = false;
            fromMinute.Enabled = false;
            fromSecond.Enabled = false;
            toDate.Enabled = false;
            toHour.Enabled = false;
            toMinute.Enabled = false;
            toSecond.Enabled = false;
            beforeDate.Enabled = false;
            beforeHour.Enabled = false;
            beforeMinute.Enabled = false;
            beforeSecond.Enabled = false;
            afterDate.Enabled = false;
            afterHour.Enabled = false;
            afterMinute.Enabled = false;
            afterSecond.Enabled = false;
        }

        /// <summary>
        /// 更新日時の抽出条件指定フィールドを活性にします。
        /// </summary>
        private void enableDateInputFields()
        {
            fromDate.Enabled = true;
            fromHour.Enabled = true;
            fromMinute.Enabled = true;
            fromSecond.Enabled = true;
            toDate.Enabled = true;
            toHour.Enabled = true;
            toMinute.Enabled = true;
            toSecond.Enabled = true;
            beforeDate.Enabled = true;
            beforeHour.Enabled = true;
            beforeMinute.Enabled = true;
            beforeSecond.Enabled = true;
            afterDate.Enabled = true;
            afterHour.Enabled = true;
            afterMinute.Enabled = true;
            afterSecond.Enabled = true;
        }

        /// <summary>
        /// 抽出結果一覧のセルがダブルクリックされた時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void dataGridView1_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            // ファイル名列がダブルクリックされた時
            if (e.ColumnIndex == 1 && e.RowIndex >= 0)
            {
                // ダブルクリックされた行のファイル名とファイルパスを取得する
                string filePath = dataGridView1.Rows[e.RowIndex].Cells[2].Value.ToString() + "\\" + dataGridView1.Rows[e.RowIndex].Cells[1].Value.ToString();

                // ファイルが実行形式の場合は、実行確認ダイアログを表示する
                if (filePath.EndsWith(".exe") || filePath.EndsWith(".bat"))
                {
                    ExecuteConfirmDialog dialog = new ExecuteConfirmDialog();
                    DialogResult result = dialog.ShowDialog();

                    if (result == DialogResult.OK)
                    {
                        Process.Start(filePath);
                    }
                }
                else if (filePath.EndsWith(".dll")) // DLLの場合
                {
                    MessageUtil.ShowAlert("このファイルは開けません。");
                }
                else
                {
                    Process.Start(filePath);
                }
            }
        }

        /// <summary>
        /// お気に入りに追加メニューがクリックされた時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void addToFavoriteMenuItem_Click(object sender, EventArgs e)
        {
            AddFavoriteDialog dialog = new AddFavoriteDialog();

            dialog.SourceFolder = extractDirectoryComboBox.Text;
            dialog.ExtractFile = extractFileComboBox.Text;
            dialog.ExcludeFile = exceptFileComboBox.Text;
            dialog.DestinationFolder = destinationDirectoryComboBox.Text;

            dialog.ShowDialog();
        }

        /// <summary>
        /// お気に入りの読み込みメニューがクリックされた時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void readFavoriteMenuItem_Click(object sender, EventArgs e)
        {
            ReadFavoriteDialog dialog = new ReadFavoriteDialog(this);

            dialog.ShowDialog();
        }

        /// <summary>
        /// お気に入りデータを抽出条件の入力欄に設定します。
        /// </summary>
        /// <param name="favorite">お気に入りデータ</param>
        public void SetFavoriteData(FavoriteData favorite)
        {
            extractDirectoryComboBox.Text = favorite.ExtractFolder;
            extractFileComboBox.Text = favorite.ExtractFile;
            exceptFileComboBox.Text = favorite.ExcludeFile;
            destinationDirectoryComboBox.Text = favorite.DestinationFolder;
        }
    }
}
